/*
 * Copyright (c) 2018, Oracle and/or its affiliates.  All rights reserved.
 *
 * This software is dual-licensed to you under the MIT License (MIT) and
 * the Universal Permissive License (UPL).  See the LICENSE file in the root
 * directory for license terms.  You may choose either license, or both.
 */

// com.oracle.iot.client.impl.device;

import Foundation

/**
 * DeviceAnalog combines a device (endpoint id and attributes) with a model.
 */
public protocol DeviceAnalog {

    /**
     * Get the endpoint id of the device
     *
     * - returns: the device endpoint id
     */
    func getEndpointId() -> String

    /**
     * Get the device model of the device.
     *
     * - returns: the device model
     */
    func getDeviceModel() -> DeviceModel

    /**
     * Set the named attribute to the given value.
     *
     * - parameter attribute: the attribute to set
     *
     * - parameter value: the value of the attribute
     *
     * - throws: `ClientError.argument` if the attribute is not in the device
     *                                model,the value is `nil`, or the 
     *                                value does not match the attribute type.
     */
    func setAttributeValue(attribute:String, value:AnyObject) throws

    /**
     * Get the value of the named attribute.
     *
     * - parameter attribute: the attribute to get
     *
     * - returns: the value of the attribute, possibly `nil`
     */
    func getAttributeValue(attribute:String) throws -> AnyObject?
    
    // TODO: call may need a callback it calls "send" which requires a
    // callback. But there are other methods that call "call" that must
    // return a value. For now just call send, which returns immeditately
    // and assume it does not affect the rest of the call implementation
    /**
     * Invoke an action.
     *
     * - parameter actionName: the name of the action to call
     *
     * - parameter args: the arguments, possibly `nil` or empty, to pass to the action
     */
    func call(actionName:String, args:[Any?])

    // TODO: this API doesn't really fit here, but needed for alertCondition
    /**
     * Queue a message for dispatch to the server. The implementation of
     * this method may send the message directly without queuing.
     *
     * - parameter message: the message to be queued
     */
    func queueMessage(message:Message) throws
}
