/*
 *  Copyright © 2016, 2017, Oracle and/or its affiliates. All rights reserved.
 *  This software is dual-licensed to you under the MIT License (MIT) and
 *  the Universal Permissive License (UPL). See the LICENSE file in the root
 *  directory for license terms. You may choose either license, or both.
 */

import Foundation
import SwiftyJSON

/**
 * IndirectActivationResponse
 */
class IndirectActivationResponse {
    
    private static let FIELD_ENDPOINT_ID: String       = "endpointId"
    private static let FIELD_ACTIVATION_TIME: String   = "activationTime"
    private static let FIELD_ENDPOINT_STATE: String    = "endpointState"
    
    private var endpointId: String?
    private var activationTime: Date?
    private var endpointState: String?
    
    init() {}
    
    func getActivationTime() -> Date {
        return activationTime!
    }
    
    func setActivationTime(activationTime: Date) {
        self.activationTime = activationTime
    }
    
    func getEndpointState() -> String {
        return endpointState!
    }
    
    func setEndpointState(endpointState: String) {
        self.endpointState = endpointState
    }
    
    func getEndpointId() -> String {
        return endpointId!
    }
    
    func setEndpointId(endpointId: String) {
        self.endpointId = endpointId
    }
    
    func toString() -> String {
        var result: String = "IndirectActivationResponse{"
        result = result + "endpointId='" + endpointId! + "'"
        result = result + ", endpointState='" + endpointState! + "'" + "}"
        return result
    }
    
    static func fromJson(data: Foundation.Data?) -> IndirectActivationResponse? {
        if let dataIn = data {
            var jsonDict:JSON!
            do {
                jsonDict = try JSON(data: dataIn)
            } catch {
                print(String(format:NSLocalizedString(
                    "%@: JSON threw exception: %@", comment:""),
                    "IndirectActivationRequest.fromJSON()", "\(error)"))
                return nil
            }
            let response: IndirectActivationResponse = IndirectActivationResponse()
            
            response.endpointId = jsonDict[FIELD_ENDPOINT_ID].string
            if(response.endpointId == nil) {
                print("Endpoint ID  is nil!")
            }
            
            // TODO: implement a Date converter
            response.endpointState = jsonDict[FIELD_ENDPOINT_STATE].string
            return response
        } else {
            return nil
        }
    }
    
    func toJsonString() -> String {
        var resultStr: String = "{"
        resultStr = resultStr + "\"" + IndirectActivationResponse.FIELD_ENDPOINT_ID + "\":\"" + endpointId! + "\","
        resultStr = resultStr + "\"" + IndirectActivationResponse.FIELD_ENDPOINT_STATE + "\":\"" + endpointState!  + "\","
        resultStr = resultStr + "\"" + IndirectActivationResponse.FIELD_ACTIVATION_TIME + "\":\"" + activationTime!.description(with: Locale.autoupdatingCurrent) + "\"}"
        return resultStr
    }
}
