/*
 *  Copyright © 2016 Oracle and/or its affiliates. All rights reserved.
 *  This software is dual-licensed to you under the MIT License (MIT) and
 *  the Universal Permissive License (UPL). See the LICENSE file in the root
 *  directory for license terms. You may choose either license, or both.
 */

import Foundation

/**
 * Abstract superclass for request handlers.
 *
 * - (Abstract class)
 *
 */
open class RequestHandler {
    public init() {
    }
    
    /**
     * Handles incoming Request message, creating a Response message.
     *
     * - (Abstract method)
     *
     * - parameter request: incoming Request message
     *
     * - returns: outgoing Response message
     *
     * - Throws: WrongSource
     */
    open func handleRequest(requestMessage: RequestMessage?) throws -> ResponseMessage? {
        fatalError("Subclass need to implement the 'handleRequest()' method.")
    }
}

class AsyncRequestHandler : RequestHandler {
    public override init() {
    }
    
    /**
     * Handles incoming Request message, creating a Response message.
     *
     * - (Abstract method)
     *
     * - parameter request: incoming Request message
     *
     * - parameter callback: called when the asynchronous processing is complete
     *                       to return the `responseMessage`.
     *
     * - Throws: WrongSource
     */
    func handleRequest(requestMessage: RequestMessage?,
                       callback: @escaping (ResponseMessage?,
                                            ClientError?)->()) throws {
            fatalError("Subclass need to implement the 'handleRequest()' method.")
    }
}
