/*
 *  Copyright © 2016, 2017, Oracle and/or its affiliates. All rights reserved.
 *  This software is dual-licensed to you under the MIT License (MIT) and
 *  the Universal Permissive License (UPL). See the LICENSE file in the root
 *  directory for license terms. You may choose either license, or both.
 */

import Foundation


/**
 * Monitor monitors data messages for started devices and processes messages.
 */

class Monitor : ThreadRunnable {
    
    
    private let MONITOR_MAX_FORMATS = Config.MONITOR_MAX_FORMATS
    private let MONITOR_POLLING_INTERVAL = Config.MESSAGE_POLLING_INTERVAL
    
    private let syncQueue:DispatchQueue = DispatchQueue.global(qos: .utility)
    private var running: Bool = false

    private var lastUntil:CLongLong?

    // Delay polling for messages in milliseconds, default 3 seconds
    private let pollingInterval: Double

    // The maximum number of alerts to return in a bulk data request.
    private let maxFormats: Int
    private var ecl: EnterpriseClient?
    
    init(ec: EnterpriseClient) {
        // this is deliberate to avoid autoboxing
        var val = Double(MONITOR_POLLING_INTERVAL)
        self.pollingInterval = (val > 0 ? val : 3000)
        
        val = Double(MONITOR_MAX_FORMATS)
        self.maxFormats = Int(val > 0 ? val : 10)
        self.ecl = ec
    }
    
    /**
     * Starts a thread monitor for an enterprise client object.
     *
     * - Parameter eClient: The enterprise client object which is monitoring.
     */
    func start() {
        syncQueue.sync(execute: {
            if !running {
                running = true
                self.backgroundThread(background: self.run, completion: self.stop)
            }
        })
    }

    private func run() {
        while running {
            self.ecl?.remoteBulkMonitor()
            sleep(UInt32(self.pollingInterval/1000))
        }
    }
        
    func stop() {
        syncQueue.sync(execute: {
            running = false
        })
    }
}
