/*
 *  Copyright © 2016, 2017, Oracle and/or its affiliates. All rights reserved.
 *  This software is dual-licensed to you under the MIT License (MIT) and
 *  the Universal Permissive License (UPL). See the LICENSE file in the root
 *  directory for license terms. You may choose either license, or both.
 */

import Foundation

/**
 * This class enables low level classes to access the HttpSecureConnection 
 * created by a EnterpriseClient.
 */
class SecureHttpConnectionMap {
    
    private static var secureConnectionMap: [EnterpriseClient : HttpSecureConnection] = [EnterpriseClient : HttpSecureConnection]()
    
    /**
     * Get the SecureConnection instance created by a client.
     *
     * - Parameter client: Creator of the connection.
     *
     * - Returns: SecureConnection or nil.
     */
    static func getSecureHttpConnection(client: EnterpriseClient) -> HttpSecureConnection {
        return secureConnectionMap[client]!
    }
    
    /**
     * Put a SecureConnection created by an EnterpriseClient in the map. If a 
     * HttpSecureConnection is nil then remove client's entry from the map.
     *
     * - Parameter client: Creator of the connection.
     *
     * - Parameter secureConnection: Connection or nil to remove the connection.
     */
    static func putSecureConnection(client: EnterpriseClient, secureConnection: HttpSecureConnection) -> Void {        
        secureConnectionMap[client] = secureConnection
    }
}
