/*
 *  Copyright © 2016, 2017, Oracle and/or its affiliates. All rights reserved.
 *  This software is dual-licensed to you under the MIT License (MIT) and
 *  the Universal Permissive License (UPL). See the LICENSE file in the root
 *  directory for license terms. You may choose either license, or both.
 */

import Foundation
// FIXME: comparison operators with optionals were removed from the Swift Standard Libary.
// Consider refactoring the code to use the non-optional operators.
fileprivate func < <T : Comparable>(lhs: T?, rhs: T?) -> Bool {
  switch (lhs, rhs) {
  case let (l?, r?):
    return l < r
  case (nil, _?):
    return true
  default:
    return false
  }
}

// FIXME: comparison operators with optionals were removed from the Swift Standard Libary.
// Consider refactoring the code to use the non-optional operators.
fileprivate func >= <T : Comparable>(lhs: T?, rhs: T?) -> Bool {
  switch (lhs, rhs) {
  case let (l?, r?):
    return l >= r
  default:
    return !(lhs < rhs)
  }
}


/**
 * HttpResponse
 */
public class HttpResponse {
    private let status: Int?
    private let data: Foundation.Data?
    private let headers: [String : [String]]?
    
    public init(responseCode: Int, data: Foundation.Data, headers: [String : [String]]) {
        self.status = responseCode
        self.data = data
        self.headers = headers
    }
    
    public func getStatus() -> Int {
        return self.status!
    }
    
    public func getData() -> Foundation.Data {
        return self.data!
    }
    
    public func getHeaders() -> [String : [String]] {
        return self.headers!
    }
    
    // TODO: FIXME throw ClientError
    public func getVerboseStatus( method: String?, path: String?) throws -> String {
        var result: String = ""
        if (method != nil) {
            result = method! + " "
        }
        if path != nil {
            result = result + path! + ": "
        }
        result = result + "HTTP " + (self.status?.description)!
        
        if status >= StatusCode.BAD_REQUEST.getCode() {
            if (data != nil) && (data!.count > 0) {
                if let dataStr = String(data: data!, encoding: .utf8) {
                    result = result + " " + dataStr
                } else {
                    throw ClientError.network(String(format: NSLocalizedString(
                        "HTTP Error in getVerboseStatus: %@" , comment: ""),
                                                     (status?.description)!))
                }
            }
        }
        
        return result
    }
}
